---
title: Tech Lunch - Intro à Rust
author: Orestis Malaspinas
date: 11 janvier 2019
sansfont: Sans Serif
pdfMaxPagesPerSlide: 
	1
<!-- slide-numbers: true -->
---
# Historique

## Une brève histoire du Rust

- Créé comme projet personnel par Graydon Hoare chez Mozilla en 2006.
- Financé par Mozilla dès 2010.
- Rendu public la première fois en 2012 (version 0.4).
- Première version stable (1.0) en 2015.
- Stabilisation de fonctionnalités tous les 6 semaines.
- Version stable actuelle 1.31.1 <!-- TODO UPDATE -->

## Un développement pragmatique

- La très grande majorité des fonctionnalités originelles ont été retirées.
- Développement du langage orienté sur la communauté: *Request For Comments*.
- Langage système et de haut niveau: *Memory safety & concurrency without fear*.

# Projets basés sur Rust

- Firefox:
	* Servo (moteur parallèle de navigateur en collaboration avec Samsung)
	* Quantum (Moteur du navigateur de Firefox)
- Gnome (en remplacement du C).
- Dropbox.
- Npm.
- Cargo.
- Tor (en phase de tests).
- Redox (microkernel).

. . .

*<https://www.rust-lang.org/en-US/friends.html>*

# Concepts de base

## Généralités

- Des garanties de sûreté
	- Pas de seg-faults.
	- Pas d'accès concurrents.
	- Un système de type expressif.
. . .
- Sans coût de performance
	- Pas de garbage collector.
	- Pas d'overhead à l'exécution.

. . .

**Un langage "système" dans lequel on peut avoir confiance.**

- Productif: développement rapide, et exécution efficace.
- Projets "plus simples" à maintenir.

. . .

*Zero-cost abstractions & memory safety & data-race free* 
<!-- Productivité: efficacité et maintenance "simple". -->

# (Non-)sûreté mémoire

## Performance: un peu de C++

:::::::::::::: {.columns}
::: {.column width="50%"}
```
{
	vector<string> v; // alloué sur la pile
					  // et l'espace est
					  // contigu en mémoire
	...
	auto& elem = v[0]; // référence
	...
} // destruction automatique
```
:::
::: {.column width="50%"}
![](figs/mem_vec_str.svg){#fig:mem_vec_str width=80%}
:::
::::::::::::::

## Et la sécurité dans tout ça?

:::::::::::::: {.columns}
::: {.column width="50%"}
```
{
	vector<string> v; // alloué sur la pile
					  // et l'espace est
					  // contigu en mémoire
	...
	auto& elem = v[0]; // référence
	v.push_back(a_string);
	cout << elem;
	...
} // destruction automatique
```
:::
::: {.column width="50%"}
![](figs/mem_vec_str_error.svg){#fig:mem_vec_str width=80%}
:::
::::::::::::::

. . .

Quel est le problème dans ce code?

. . .

**`elem` est un pointeur pendouillant!**

## D'où provient le problème?

- Deux contraintes doivent être satisfaites en même temps
	1. Aliasing à l'élément 0 de `vector`.
	2. Modification de `vector` (désalloué et réalloué ailleurs).

- Simple de suivre *une* référence, difficile de les mettre à jour toutes.

# Sûreté mémoire en Rust

## Ownership (1/1)

- Quand on est **propriétaire** des données, on peut faire ce qu'on veut avec:
	1. Les lire.
	2. Les modifier.
	3. Les donner à quelqu'un d'autre.

## Ownership (2/2)

![Comme avec un cahier. Source [Wikipedia](https://upload.wikimedia.org/wikipedia/commons/9/9f/Se%C5%A1ity.JPG)](https://upload.wikimedia.org/wikipedia/commons/9/9f/Se%C5%A1ity.JPG){#fig:mem_vec_str width=50%}

## Ownership en pratique

<pre><code data-trim="hljs rust" class="lang-rust">
    fn do_something(_notebook: Vec<&str>) { // _notebook prend l'ownership
    // Do something with the library
} // _notebook est détruit et ses données aussi

fn main() {
    let mut notebook = Vec::new(); // par défaut notebook est immutable

    notebook.push("Il fait beau.");
    notebook.push("Les oiseaux chantent.");

    do_something(notebook); // ownership transféré à do_something
    // do_something(notebook); // redo something
}
</code></pre>

## Ownership en pratique

Ce qui se passe en mémoire:

![](figs/mem_vec_ownership.svg){#fig:mem_vec_ownership width=45%}![](figs/mem_vec_ownership_move.svg){#fig:mem_vec_ownership_move width=45%}

![](figs/mem_vec_ownership_dest.svg){#fig:mem_vec_ownership_move width=45%}

. . .

**`notebook` est détruit à la fin de `do_something` et ne peut plus être réutilisé**



<!-- - `vector` et `string` sont alignés en mémoire.
- références "light".
- destruction déterministe. -->

<!-- ## Généralités

- Fortement typé.
- Pas d'accès mémoire illégal.
- La désallocation est automatique.
- Les fuites de mémoires sont TRÈS difficiles à faire.

# Concurrence

- *Concurrency without fear*.
- Le système de type empêche un accès concurrent à des données.
- La synchronisation des données doit être explicitement faite.
- Détection si l'accès asynchrone est sûr à la compilation.
- Protection contre les accès concurrents. 

## Rapidité

- Abstractions à coût nul (langage moderne **ET** efficace).
- Les conditions de sécurité sont gérées *à la compilation* (pas de coût à l'exécution).
- Le compilateur est basé sur la LLVM.
- On paie pour ce qu'on utilise seulement.

## On peut contourner les règles

- Le compilateur a des règles très strictes...
- ... mais il est possible de contourner les règles.
- Syntaxe spéciale pour autoriser les comportement potentiellement dangereux (`unsafe`{.lang-rust}).
- Un guide pour bien vivre dangereusement <https://doc.rust-lang.org/nomicon/>. -->

